(* OCaml rewrite of Vim's xxd to convert any file in a C header file containing
 * its hex values.
 * In contrast to Vim's xxd, a null terminated array is produced to allow the
 * usage of strlen. When using the _len variable, one encounters no difference.
 *)

#load "str.cma";;

let rec read_chars sf =
  try
    let c = int_of_char (input_char stdin) in
    read_chars (c::sf)
  with End_of_file ->
    List.rev sf
;;

let replace find replace s =
  Str.global_replace (Str.regexp_string find) replace s
;;

let escape_str =
  replace "." "_"
;;

let _ =
  let l = read_chars [] in
  let fn = escape_str (Sys.argv.(1)) in
  Format.printf "unsigned char %s[] = {@;<1 2>@[" fn;
  List.iter (Format.printf "0x%02x,@ ") l;
  Format.printf "0x00@.};@.";
  Format.printf "unsigned int %s_len = %d;@." fn (List.length l);
  ()
;;

